#include "stdafx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "interfaces.h"
#include "manager.h"
#include "xcamera.h"
#include "XTexture.h"
#include "XMesh.h"
#include "XVertexBuffer.h"
#include "XIndexBuffer.h"
#include "xresourcemanager.h"
#include "xdisplay.h"
#include "x3dobject.h"

X3DObject::X3DObject(void)
{
	m_lpDevice = NULL;	
	m_lpTexture = NULL;
	m_lpIB = NULL;
	m_lpVB = NULL;
	D3DXMatrixIdentity(&m_TransMat);
	m_dwRef = 0;
}

X3DObject::~X3DObject(void)
{
	ReleaseResource();	
	SAFE_RELEASE(m_lpTexture);
	SAFE_RELEASE(m_lpIB);
	SAFE_RELEASE(m_lpVB);
}

HRESULT		X3DObject::AddRef()
{
	return m_dwRef++;
}

HRESULT		X3DObject::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;		
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("X3DObject object destroyed.");
		return 0;
	}	
	else
	{
		return m_dwRef;
	}		
}

int		X3DObject::ReleaseResource()
{
	if(m_lpDevice) 
	{
		m_lpTexture->ReleaseResource();
		m_lpVB->ReleaseResource();
		m_lpIB->ReleaseResource();
	}
	return S_OK;
}
// standard methods
int						X3DObject::Init( 
										LPCSTR szTexture, int iMipMap, D3DFORMAT formatTexture,  
										DWORD dwVBSize, 
										DWORD dwIBSize, 
										DWORD dwFlags)
{
	DWORD dwUsage = 0;
	int iRet;
	IDisplay *dis = NULL;
	CreateDisplayObject(DISIID_IDisplay, (void**) &dis);
	if(!dis)
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}
	// Get device from IDisplay
	m_lpDevice = dis->GetDevice();
	SAFE_RELEASE(dis);
	if(m_lpDevice)
	{
		if(dwVBSize != 0)
		{	
			if(VB_WRITEONLY & dwFlags)
			{
				dwUsage |= D3DUSAGE_WRITEONLY;
			}
			CreateDisplayObject(DISIID_IVertexBuffer, (void**)&m_lpVB);
			iRet = m_lpVB->Create(dwVBSize, dwUsage, VERTEXFORMAT, D3DPOOL_DEFAULT);
			if(iRet != 0)
			{
				TRACE("Cannot create VB for 3DObject.");
			}
		}
		if(dwIBSize != 0)
		{			
			dwUsage = 0;
			if(IB_WRITEONLY & dwFlags)
			{
				dwUsage |= D3DUSAGE_WRITEONLY;
			}
			CreateDisplayObject(DISIID_IIndexBuffer, (void**)&m_lpIB);
			iRet = m_lpIB->Create(dwIBSize, dwUsage, D3DFMT_INDEX16, D3DPOOL_DEFAULT);
			if(iRet != 0)
			{
				TRACE("Cannot create IB for 3DObject.");
			}
		}
		if(szTexture)
		{			
			CreateDisplayObject(DISIID_ITexture, (void**)&m_lpTexture);
			iRet = m_lpTexture->LoadTextureFromFileEx(szTexture, iMipMap, formatTexture);
			if(iRet != 0)
			{
				TRACE("Cannot create texure.");
			}
		}
		m_bVisible = TRUE;
		m_bApplyTransformations = TRUE;
		return S_OK;
	}
	return S_FALSE;
}
int						X3DObject::Draw(D3DPRIMITIVETYPE Type,
										UINT MinIndex,
										UINT NumVertices,
										UINT StartIndex,
										UINT PrimitiveCount)
{	
	if(m_lpDevice)
	{	
		if(&m_TransMat)
		{			
			m_lpDevice->SetTransform(D3DTS_WORLD, &m_TransMat);	
		}
		if(m_bVisible)
		{			
			m_lpDevice->SetVertexShader(VERTEXFORMAT);
			m_lpDevice->SetStreamSource(0, m_lpVB->GetBuffer(), sizeof(VERTEX));
			m_lpDevice->SetIndices(m_lpIB->GetBuffer(), 0);
			m_lpDevice->SetTexture(0, m_lpTexture->GetTexture());
			m_lpDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 4, 0, 2);	
		}
		// set indentity matrix
		if(&m_TransMat)
		{
			D3DXMATRIX mat;
			D3DXMatrixIdentity(&mat);
			m_lpDevice->SetTransform(D3DTS_WORLD, &mat);
		}
		return S_OK;
	}
	return S_FALSE;
}

int						X3DObject::Restore()
{
	if(m_lpDevice) 
	{
		m_lpTexture->Restore();
		m_lpVB->Restore();
		m_lpIB->Restore();
		// after that, caller must refill buffers
		return S_OK;
	}
	return S_FALSE;
}

void	X3DObject::Translate(float fX, float fY, float fZ)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create translation matrix
		D3DXMatrixTranslation(&matTemp, fX, fY, fZ);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(&m_TransMat, &m_TransMat, &matTemp);
	}
}

void	X3DObject::RotateX(float fAngle)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// create rotation matrix
		D3DXMatrixRotationX(&matTemp, fAngle);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(&m_TransMat, &m_TransMat, &matTemp);
	}
}

void	X3DObject::RotateY(float fAngle)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// create rotation matrix
		D3DXMatrixRotationY(&matTemp, fAngle);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(&m_TransMat, &m_TransMat, &matTemp);
	}
}

void	X3DObject::RotateZ(float fAngle)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// create rotation matrix
		D3DXMatrixRotationZ(&matTemp, fAngle);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(&m_TransMat, &m_TransMat, &matTemp);
	}
}

void	X3DObject::ScaleXYZ(float fFactor)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create scaling matrix
		D3DXMatrixScaling(&matTemp, fFactor, fFactor, fFactor);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(&m_TransMat, &m_TransMat, &matTemp);
	}
}

void	X3DObject::ScaleX(float fFactor)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create scaling matrix
		D3DXMatrixScaling(&matTemp, fFactor, 1.0f, 1.0f);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(&m_TransMat, &m_TransMat, &matTemp);
	}
}

void	X3DObject::ScaleY(float fFactor)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create scaling matrix
		D3DXMatrixScaling(&matTemp, 1.0f, fFactor, 1.0f);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(&m_TransMat, &m_TransMat, &matTemp);
	}
}

void	X3DObject::ScaleZ(float fFactor)
{
	if(m_bApplyTransformations)
	{
		D3DXMATRIX	matTemp;
		// Create scaling matrix
		D3DXMatrixScaling(&matTemp, 1.0f, 1.0f, fFactor);
		// modify transformation matrix of the object
		D3DXMatrixMultiply(&m_TransMat, &m_TransMat, &matTemp);
	}
}