#pragma once

#define		VB_WRITEONLY		0x00000001
#define		IB_WRITEONLY		0x00000002

class X3DObject : public I3DObject
{
	// is object visible?
	BOOL					m_bVisible;
	// set trans matrix before render?
	BOOL					m_bApplyTransformations;
	// vertex buffer
	IVertexBuffer*			m_lpVB;
	// index buffer
	IIndexBuffer*			m_lpIB;
	// texture
	ITexture	*			m_lpTexture;
	// transformation matrix
	D3DXMATRIX				m_TransMat;
	// pointer to device
	LPDIRECT3DDEVICE8		m_lpDevice;

	DWORD		m_dwRef;
	
public:
	// standard methods
	virtual int						Init(	 
									LPCSTR szTexture, int iMipMap, D3DFORMAT formatTexture ,
									DWORD dwVBSize, 
									DWORD dwIBSize, 
									DWORD dwFlags);
	virtual int						Draw(	D3DPRIMITIVETYPE Type,
									UINT MinIndex,
									UINT NumVertices,
									UINT StartIndex,
									UINT PrimitiveCount);
	virtual int						Restore();	
	virtual int						ReleaseResource();
	virtual IVertexBuffer*			GetVB()			{ return m_lpVB; }
	virtual IIndexBuffer*			GetIB()			{ return m_lpIB; }
	virtual ITexture*				GetTexture()	{ return m_lpTexture; }
	virtual LPDIRECT3DDEVICE8		GetDevice()		{ return m_lpDevice; }
	virtual BOOL					IsVisible()		{ return m_bVisible; }
	virtual void					Visible(BOOL bVis = TRUE) { m_bVisible = bVis; }
	virtual void					EnableTransformation(BOOL bEnable = TRUE) {m_bApplyTransformations = bEnable;}
	virtual BOOL					IsTransformationEnabled() { return m_bApplyTransformations;}

	// transformations methods
	virtual void					Translate(float fX, float fY, float fZ);
	virtual void					RotateX(float fAngle);
	virtual void					RotateY(float fAngle);
	virtual void					RotateZ(float fAngle);
	virtual void					ScaleXYZ(float fFactor);
	virtual void					ScaleX(float fFactor);
	virtual void					ScaleY(float fFactor);
	virtual void					ScaleZ(float fFactor);	

	virtual HRESULT		AddRef();
	virtual HRESULT		Release();
public:
	X3DObject(void);
	~X3DObject(void);
};
