#include "StdAfx.h"

#define		DISPLAY_API	__declspec(dllexport)

#include "../Common/Common.h"
#include "interfaces.h"
#include "manager.h"
#include "xfont.h"
#include "xmesh.h"
#include "xvertexbuffer.h"
#include "xindexbuffer.h"
#include "xresourcemanager.h"

XFont::XFont(void)
{
	m_dwRef = 0;
	m_theFont = NULL;
}

XFont::~XFont(void)
{
	SAFE_RELEASE(m_theFont);
}

HRESULT		XFont::AddRef()
{
	return m_dwRef++;
}

HRESULT		XFont::Release()
{
	if(m_dwRef > 0)
	{
		m_dwRef--;
	}
	if(m_dwRef == 0)
	{			
		delete this;
		TRACE("XFont object destroyed.");
		return 0;
	}	
	else
	{
		return m_dwRef;
	}	
}

HRESULT XFont::CreateFont(int iSize, int iWidth, LPCSTR szFamily, BOOL iBold, D3DCOLOR dColor)
{
	DWORD	dwRet = S_FALSE;
	if(m_theFont)
	{	
		TRACE("Font is already inicialized.");
		return ERROR_INVALID_PARAMETER;
	}
	IDisplay *dis = NULL;	
	CreateDisplayObject(DISIID_IDisplay, (void**) &dis);
	if(!dis)
	{
		XException exp("LoadTextureFromFile: Device is invalid.");
		THROW(exp);			
	}
	// Get device from IDisplay
	m_lpDevice = dis->GetDevice();	

	// Create windows font
	HFONT hFont = ::CreateFont(iSize, iWidth, 0, 0, (iBold) ? FW_BOLD : 0, 0, 0, 0, ANSI_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 0, FF_DONTCARE|DEFAULT_PITCH , szFamily);
	if(hFont)
	{
		// Create d3d font
		if(D3D_OK == D3DXCreateFont(m_lpDevice, hFont, &m_theFont))
		{			
			dis->GetResourceManager()->AddFont(this);
			m_dColor = dColor;		
			m_theFont->GetLogFont(&m_lg);
			dwRet = S_OK;
		}
		else
		{
			TRACE("Cannot create D3D font.");
			dwRet = S_FALSE;
		}		
	}	
	else
	{
		TRACE("Cannot create Windows font.");
		dwRet = S_FALSE;
	}
	SAFE_RELEASE(dis);

	return dwRet;
}

HRESULT	XFont::Draw(LPCSTR szText, int x, int y, D3DCOLOR dColor/* = -1*/)
{
	if(!m_theFont) 
	{
		TRACE("Font is not initialized.");
		return ERROR_INVALID_PARAMETER;
	}
	RECT rcBase = {x, y, 0, 0};
	// Draw a text		
	m_theFont->DrawText(szText, -1, &rcBase, DT_CALCRECT, 0);		
	m_theFont->DrawText(szText, -1, &rcBase, DT_LEFT|DT_WORDBREAK, (dColor == -1) ? m_dColor : dColor);		
	return S_OK;
}


int		XFont::Restore()
{
	if(m_theFont) return m_theFont->OnResetDevice();
	return S_FALSE;
}

int		XFont::ReleaseResource()
{
	if(m_theFont) return m_theFont->OnLostDevice();
	return S_FALSE;
}
