/*
 *
 * Manager.cpp - Implementation of manager
 * Date:	4.6.2006
 * Update:	-
 * Author:	Jiri Formanek
 *
 */
#include "StdAfx.h"
#define DISPLAY_API __declspec(dllexport)

#include "../Common/Common.h"
#include "Interfaces.h"
#include "Manager.h"
#include "XTexture.h"
#include "XIndexBuffer.h"
#include "XVertexBuffer.h"
#include "XMesh.h"
#include "XFont.h"
#include "X3dobject.h"
#include "XResourceManager.h"
#include "XCamera.h"
#include "XDisplay.h"

// Call This function to get desired interface
DISPLAY_API  HRESULT	CreateDisplayObject(DISIID InterfaceID, void ** ppv)
{
	// unique objects
	static IDisplay		* g_theDisplay = NULL;	
	static ICamera		*	g_theCamera = NULL;	
	static IResourceManager		* g_theResourceManager = NULL;	
	if(!ppv)
	{
		return ERROR_INVALID_PARAMETER;
	}
	// get pointer according IID 
	switch(InterfaceID) {
		case DISIID_IDisplay:
			if(!g_theDisplay) 
			{
				*ppv = g_theDisplay = new XDisplay;
			}
			else {
				*ppv = g_theDisplay;
			}
			((IDisplay*)(*ppv))->AddRef();
			break;
		case DISIID_ICamera:
			if(!g_theCamera) 
			{
				*ppv = g_theCamera = new XCamera;
			}
			else {
				*ppv = g_theCamera;
			}
			((ICamera*)(*ppv))->AddRef();
			break;
		case DISIID_IResourceManager:
			if(!g_theResourceManager) 
			{
				*ppv = g_theResourceManager = new XResourceManager;
			}
			else {
				*ppv = g_theResourceManager;
			}
			((IResourceManager*)(*ppv))->AddRef();
			break;
		case DISIID_IVertexBuffer:				
			*ppv = new XVertexBuffer;			
			((IVertexBuffer*)(*ppv))->AddRef();
			break;
		case DISIID_IIndexBuffer:				
			*ppv = new XIndexBuffer;			
			((IIndexBuffer*)(*ppv))->AddRef();
			break;
		case DISIID_IMesh:				
			*ppv = new XMesh;			
			((IMesh*)(*ppv))->AddRef();
			break;
		case DISIID_ITexture:				
			*ppv = new XTexture;			
			((ITexture*)(*ppv))->AddRef();
			break;
		case DISIID_I3DObject:				
			*ppv = new X3DObject;			
			((I3DObject*)(*ppv))->AddRef();
			break;
		case DISIID_I3DFont:				
			*ppv = new XFont;			
			((I3DFont*)(*ppv))->AddRef();
			break;
		default:
			return E_NOINTERFACE;
	}
	return S_OK;
}