// Tester2.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "Tester2.h"

#include "../common/common.h"
#include "../display/common.h"
#include "../input/common.h"

#include "terrain.h"

#define		_S_DEBUG			"Common"
#define		_S_TRACETOFILE		"TraceToFile"


// Global Variables:
HINSTANCE hInst;								// current instance

IDisplay	*g_pDisplay = NULL;
IInput		*g_pInput = NULL;
I3DFont		*g_pKeys = NULL;

CTerrain	g_theTerrain;

D3DLIGHT8		g_theSun;

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int, HWND *hWnd);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
void				Exit();
void				UpdateFrame();
void				RestoreUserObjects();

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{ 	
	MSG msg;
	char szHeightMap[MAX_PATH];
	DWORD dwRet = S_OK;
	srand(GetTickCount());

	

	MyRegisterClass(hInstance);

	HWND hWnd = NULL;
	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow, &hWnd)) 
	{
		return FALSE;
	}
	// inicializace logovaciho souboru, pokud je zapnuta funkce Tracetofile
	if(cmnReadSetupInt(_S_DEBUG, _S_TRACETOFILE) == 1) {
		cmnInitLog("log.txt", FALSE);
	}
	
	//
	// try to create IDisplay object	
	if(S_OK == (dwRet = CreateDisplayObject(DISIID_IDisplay, (void**) &g_pDisplay)))
	{
		try {
			
			g_pDisplay->Init(hWnd);	
			if(S_OK == (dwRet = CreateDisplayObject(DISIID_I3DFont, (void**) &g_pKeys)))
			{
				g_pKeys->CreateFont(15, 8, "Arial", TRUE, D3DCOLOR_ARGB(255,255,200,200));
			}
			else
			{
				TRACE("Cannot create font: %d", dwRet);
				Exit();
				return dwRet;
			}
			// read terrain method from ini file	 
			TERRAIN_METHOD iTerrainMethod = (TERRAIN_METHOD)cmnReadSetupInt(_S_TERRAIN, _S_METHOD);				
			if(iTerrainMethod == TM_HEIGHTMAP)
			{
				if(cmnReadSetupString(_S_TERRAIN, _S_HEIGHTMAP, szHeightMap, MAX_PATH) < 1)
				{
					strcpy(szHeightMap, "default.bmp");					
				}
				g_theTerrain.GenerateTerrainFromFile(szHeightMap);
			}
			else
			{
				g_theTerrain.GenerateTerrain(256, 256, iTerrainMethod);
			}
			
			g_pDisplay->GetCamera()->InitCamera(0, g_theTerrain.GetTitlesX());			

			ZeroMemory( &g_theSun, sizeof(D3DLIGHT8) );
			g_theSun.Type = D3DLIGHT_DIRECTIONAL;
			g_theSun.Direction.x = -1.0f;
			g_theSun.Direction.y = -1.0f;
			g_theSun.Direction.z = -5.0f;
			g_theSun.Diffuse.r  = 1.0f;
			g_theSun.Diffuse.g  = 1.0f;
			g_theSun.Diffuse.b  = 1.0f;
			g_theSun.Ambient.r  = 0.3f;
			g_theSun.Ambient.g  = 0.3f;
			g_theSun.Ambient.b  = 0.3f;			
			g_pDisplay->SetLight(0, &g_theSun);
			g_pDisplay->EnableLight(0, TRUE);

		}
		catch(XException exp)
		{			
			exp.msgbox();			
			Exit();
			return exp.code();
		}			
	}
	else
	{
		TRACE("Fatal error: Cannot create IDisplay object: %d",dwRet);
		return dwRet;
	}

	/* Init DI system */
	if(S_OK == (dwRet = CreateInputObject(INPIID_IInput, (void**) &g_pInput)))
	{
		try {
			g_pInput->Init(hInstance, hWnd, g_pDisplay->GetResolution());
		}
		catch(XException exp)
		{
			exp.msgbox();
			Exit();
			return exp.code();
		}	
	}
	else
	{
		TRACE("Fatal error: Cannot create IInput object: %d",dwRet);
		return dwRet;
	}

	// Main message loop:
	while( TRUE )
    {
        // Look for messages, if none are found then 
        // update the state and display it
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                // WM_QUIT was posted, so exit				
				break;                
            }
			
            // Translate and dispatch the message
            TranslateMessage( &msg ); 
            DispatchMessage( &msg );
        }
        else
        {
			// Obnova snimku
			UpdateFrame();			
		}
    }	
	Exit();
	return (int) msg.wParam;
}

void	Exit()
{
	g_theTerrain.DestroyTerrain();

	SAFE_RELEASE(g_pDisplay);
	SAFE_RELEASE(g_pInput);
	SAFE_RELEASE(g_pKeys);
}


//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage are only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_TESTER2);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= "D3DEngine Window Class";
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

//
//   FUNCTION: InitInstance(HANDLE, int)
//
//   PURPOSE: Saves instance handle and creates main window
//
//   COMMENTS:
//
//        In this function, we save the instance handle in a global variable and
//        create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow, HWND *hWnd)
{
   hInst = hInstance; // Store instance handle in our global variable

   *hWnd = CreateWindow("D3DEngine Window Class", "D3DEngine 2", WS_OVERLAPPEDWINDOW ,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!*hWnd)
   {
      return FALSE;
   }

   ShowWindow(*hWnd, nCmdShow);
   UpdateWindow(*hWnd);

   return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{		
	switch (message) 
	{
	case WM_KEYDOWN:    
        switch( wParam )
        {
		case VK_F1:
			g_pDisplay->RestoreDisplay();
			RestoreUserObjects();
			break;
        case VK_ESCAPE:
			PostQuitMessage(1);
			break;		
		}
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

void	UpdateFrame()
{
	if(g_pDisplay && g_pInput)
	{
		float fFactor = cmnGetTime(TIMER_GETELAPSEDTIME1);

		g_pDisplay->UpdateBackground();
		g_pInput->ProcessInput();

		if(g_pInput->IsKeyDown(DIK_ADD , FALSE)) {
		g_pDisplay->GetCamera()->ZoomIn();
		}
		if(g_pInput->IsKeyDown(DIK_SUBTRACT , FALSE)) {
			g_pDisplay->GetCamera()->ZoomOut();
		}
		if(g_pInput->IsKeyDown(DIK_NUMPAD1  , FALSE)) {
			g_pDisplay->GetCamera()->RotateLeft();
		}
		if(g_pInput->IsKeyDown(DIK_NUMPAD3  , FALSE)) {
			g_pDisplay->GetCamera()->RotateRight();
		}
		if(g_pInput->IsKeyDown(DIK_NUMPAD8  , FALSE)) {
			g_pDisplay->GetCamera()->IncreaseZAngle();
		}
		if(g_pInput->IsKeyDown(DIK_NUMPAD2  , FALSE)) {
			g_pDisplay->GetCamera()->DecreaseZAngle();
		}
		if(g_pInput->IsKeyDown(DIK_F, TRUE)) 
		{
            g_theTerrain.InvertFlag(TF_FROZECULLING);
		}
		if(g_pInput->IsKeyDown(DIK_M, TRUE)) 
		{
            g_theTerrain.InvertFlag(TF_DRAWQUADSMAP);
		}

		int iMouseZone = int(g_pDisplay->GetResolution()->x * 5 / 1024); 
		if(g_pInput->IsKeyDown(DIK_UP, FALSE)) {
			g_pDisplay->GetCamera()->MoveUp();
		} else {		
		if(g_pInput->GetCursor()->y < iMouseZone) {
			g_pDisplay->GetCamera()->MoveUp();			
		}}
		if(g_pInput->IsKeyDown(DIK_LEFT  , FALSE)) {
			g_pDisplay->GetCamera()->MoveLeft();
		} else {		
		if(g_pInput->GetCursor()->x < iMouseZone) {
			g_pDisplay->GetCamera()->MoveLeft();		
		}}
		iMouseZone += int( g_pDisplay->GetResolution()->x * 32 / 1024); 
		if(g_pInput->IsKeyDown(DIK_DOWN  , FALSE)) {
			g_pDisplay->GetCamera()->MoveDown();		
		} else {
		if(g_pInput->GetCursor()->y >=  g_pDisplay->GetResolution()->y-iMouseZone) {
			g_pDisplay->GetCamera()->MoveDown();		
		}}
		if(g_pInput->IsKeyDown(DIK_RIGHT  , FALSE)) {
			g_pDisplay->GetCamera()->MoveRight();	
		} else {
		if(g_pInput->GetCursor()->x >= g_pDisplay->GetResolution()->x-iMouseZone) {
			g_pDisplay->GetCamera()->MoveRight();		
		}}
	
		g_pDisplay->GetDevice()->BeginScene();
		g_theTerrain.Render();

		g_pDisplay->GetCamera()->ProcessCamera(fFactor);
		g_pKeys->Draw("Arrows - CameraMovement\n+ ZoomIn\n- ZoomOut\nNumpad5 - CameraUp\nNumpad2 - CameraDown\nNumpad1 - RotateLeft\nNumpad3 - RotateRight\nF1 - device reset\nF - froze culling\nM - shows quad tree map", 0, 100);
		
		g_pInput->UpdateCursor();		
		g_pDisplay->GetDevice()->EndScene();
		if(D3DERR_DEVICENOTRESET == g_pDisplay->Present())
		{
			RestoreUserObjects();		
		}
	}	
}

void RestoreUserObjects()
{
	g_theTerrain.Restore();
	g_pDisplay->SetLight(0, &g_theSun);
	g_pDisplay->EnableLight(0, TRUE);
}
