#ifndef BINSTROM_H
  #define BINSTROM_H

  template<class T> class CUzel {
		private:
			T m_Data;
			CUzel<T> *m_lpLevy;
			CUzel<T> *m_lpPravy;
		public:
			CUzel() : m_lpLevy(NULL), m_lpPravy(NULL) { }
			CUzel(const T& _Data) : m_Data(_Data), m_lpLevy(NULL), m_lpPravy(NULL) { }

			T VratData() const { return m_Data; }
			CUzel<T>* &VratLevy() { return m_lpLevy; }
			CUzel<T>* &VratPravy() { return m_lpPravy; }
			void NastavLevy(CUzel<T> *_lpLevy) { m_lpLevy = _lpLevy; }
			void NastavPravy(CUzel<T> *_lpPravy) { m_lpPravy = _lpPravy; }
			void NastavData(const T& _Data) { m_Data = _Data; }
  };

  template<class T> class CBinStrom {
		private:
			CUzel<T> *m_lpKoren;
		public:
			CBinStrom() : m_lpKoren(NULL) { }
			~CBinStrom() { if(!JePrazdny()) { SmazStrom(m_lpKoren); } }

			bool VytvorPrazdny();

			bool SmazStrom(CUzel<T>* &_lpUzel);

			bool JePrazdny();

			bool Vloz(const T& _Data);	// vklada do stromu
			bool VlozDoStromu(CUzel<T>* &_lpUzel, const T& _Data);
			
			CUzel<T>* Najdi(const T& _Data);	// vrati ukazatel na nalezeny prvek nebo NULL
			CUzel<T>* NajdiVeStromu(CUzel<T> *_lpUzel, const T& _Data);
			
			bool Smaz(const T& _Data);	// smaze data
  };

  template<class T> bool CBinStrom<T>::JePrazdny()
  { 
	  return (m_lpKoren == NULL);
  }

  template<class T> bool CBinStrom<T>::VytvorPrazdny()
  {
	  bool bNavrat = false;

	  if(!JePrazdny())
	  {
		  bNavrat = SmazStrom(m_lpKoren);	// smazeme puvodni strom
	  }

	  // nova inicializace promennych se provede jiz v metode SmazStrom()
	  
	  return bNavrat;
  }

  template<class T> bool CBinStrom<T>::SmazStrom(CUzel<T>* &_lpUzel)	// budeme menit ukazatel
  {
	  bool bNavrat = false;

	  if(_lpUzel)
	  {
		  SmazStrom(_lpUzel->VratLevy());
		  SmazStrom(_lpUzel->VratPravy());

		  cout << "Mazu " << (_lpUzel)->VratData() << endl;

		  delete (_lpUzel);
		  _lpUzel = NULL;
		  
		  bNavrat = true;
	  }

	  return bNavrat;
  }

  template<class T> bool CBinStrom<T>::VlozDoStromu(CUzel<T>* &_lpUzel, const T& _Data)
  {
	  bool bNavrat = false;

	  // budeme hledat spravne misto pro umisteni uzlu a ukladat si posledni nadrazeny prvek
	  CUzel<T> *lpRodic = NULL, *lpNovy = _lpUzel;

	  while(lpNovy != NULL)
	  {
		  if(lpNovy->VratData() == _Data) { return true; }	// pokud nalezneme prvek, pak koncime
		  else
		  {
			  if(lpNovy->VratData() > _Data)
			  {
				  lpRodic = lpNovy;
				  lpNovy = lpNovy->VratLevy();
			  }
			  else
			  {
				  lpRodic = lpNovy;
				  lpNovy = lpNovy->VratPravy();
			  }
		  }
	  }

	  // nyni mame misto pro novy prvek a tak ho vytvorime
	  lpNovy = new CUzel<T>(_Data);

	  if(lpNovy)
	  {
		  if(lpRodic == NULL)
		  {
			  _lpUzel = lpNovy;	// znaci, ze neexistoval prvni uzel

			  bNavrat = true;
		  }
		  else
		  {
			  if(lpRodic->VratData() > _Data)
			  {
				  lpRodic->NastavLevy(lpNovy);	// zapojime doleva
			  }
			  else
			  {
				  lpRodic->NastavPravy(lpNovy);	// zapojime doprava
			  }

			  bNavrat = true;
		  }
	  }
	  
	  return bNavrat;
  }

  template<class T> bool CBinStrom<T>::Vloz(const T& _Data)
  {
	  return VlozDoStromu(m_lpKoren, _Data);
  }

  template<class T> CUzel<T>* CBinStrom<T>::NajdiVeStromu(CUzel<T> *_lpUzel, const T& _Data)
  {
	  CUzel<T> *lpRet = NULL;

	  if(_lpUzel)
	  {
		  if(_lpUzel->VratData() == _Data)
		  {
			  return _lpUzel;
		  }
		  else
		  {
			  if(_lpUzel->VratData() < _Data)
			  {
				  lpRet = NajdiVeStromu(_lpUzel->VratPravy(), _Data);
			  }
			  else
			  {
				  lpRet = NajdiVeStromu(_lpUzel->VratLevy(), _Data);
			  }
		  }
	  }

	  return lpRet;
  } 

  template<class T> CUzel<T>* CBinStrom<T>::Najdi(const T& _Data)
  {
	  return NajdiVeStromu(m_lpKoren, _Data);
  }

#endif